@@ -30,7 +30,9 @@
 #include "libxnee/print.h"
 #include "libxnee/xnee_record.h"
 #include "libxnee/xnee_replay.h"
-
+#include "libxnee/xnee_km.h"
+#include "libxnee/xnee_fake.h"
+#include "libxnee/xnee_setget.h"
 
 /**************************************************************
  *                                                            *
@@ -42,10 +44,23 @@ void
 xnee_record_dispatch2 (XPointer xpoint_xnee_data,
 		       XRecordInterceptData *data )
 {
-  xnee_data           *xd ;
-  xd = (xnee_data *) (xpoint_xnee_data) ;
-  xd->data_fp (xd->out_file, "."); fflush (stdout);
+
+  if ( (!data->data) || (data==NULL) )
+    {
+      XRecordFreeData(data);
+      XNEE_DEBUG ( (stderr ," <-- xnee_human_dispatch()  \n"  ));
+      xnee_process_count(XNEE_PROCESS_RESET);
+      return;
+    } 
+
+  /*increment the process reply counter*/
+  xnee_process_count(XNEE_PROCESS_INC);
+  xpoint_xnee_data=0;
+  printf( "."); 
+
+  fflush (stdout);
   XRecordFreeData(data);
+  return;
 }
 
 
@@ -184,15 +199,15 @@ xnee_record_handle_event ( xnee_data *xd, XRecordInterceptData *xrecintd)
 int
 handle_xerr(Display *dpy, XErrorEvent *errevent)
 {
-int protocol_error;	/* error code of error caught by error handler */
-int protocol_error_major;	/* major op code of error caught */
-int protocol_error_minor;	/* minor op code of error caught */
-
-    protocol_error = errevent->error_code;
-    protocol_error_major = errevent->request_code;
-    protocol_error_minor = errevent->minor_code;
-    printf ("Error recevied....\n");
-    return 1;
+  int protocol_error;	        /* error code of error caught by error handler */
+  int protocol_error_major;	/* major op code of error caught */
+  int protocol_error_minor;	/* minor op code of error caught */
+  
+  protocol_error = errevent->error_code;
+  protocol_error_major = errevent->request_code;
+  protocol_error_minor = errevent->minor_code;
+  printf ("Error recevied.... on display=%d\n", (int)dpy);
+  return 1;
 }
 
 
@@ -213,53 +228,42 @@ xnee_record_dispatch(XPointer xpoint_xnee_data,
   int                  data_type = 0;  
 
   XNEE_DEBUG ( (stderr ," --> xnee_record_dispatch()  \n"  ));
-
-  if (!data->data)
+  
+  if ( (!data->data) || (data==NULL) )
     {
       XRecordFreeData(data);
-      XNEE_DEBUG ( (stderr ," <-- xnee_record_dispatch()  \n"  ));
+      XNEE_DEBUG ( (stderr ," <-- xnee_human_dispatch()  \n"  ));
+      xnee_process_count(XNEE_PROCESS_RESET);
       return;
+    } 
+  else
+    {
+      /*increment the process reply counter*/
+      xnee_process_count(XNEE_PROCESS_INC);
     }
   
-  
   /* fix by Ton van Vliet */
   xd = (xnee_data *) (xpoint_xnee_data) ;
   xrec_data = (XRecordDatum *) (data->data) ;
 
-  /* make sure we haven't been interupted by a
-     a key + modifier combination */
-  if (xnee_check_km(xd)==XNEE_GRAB_DATA)
-    {
-      XRecordFreeData(data);
-      XRecordDisableContext(xd->control, 
-			    xd->record_setup->rContext);
-      xnee_stop_session(xd);
-      return ;
-    }
 
-  
+
   if ( xd->xnee_info->loops_left == 0 ) 
     {
       XRecordFreeData(data);
-      XRecordDisableContext(xd->control, 
-			    xd->record_setup->rContext);
-      xnee_stop_session(xd);
       return ;
     } 
-  
-  XNEE_DEBUG ( (stderr ,"  -- xnee_record_dispatch() at 2 \n"  ));
-  
+  else
+    {
+      xd->xnee_info->loops_left--;
+    }
+
+ 
   /* if NOT XRECORDEndOffData or XRECORDClientDied  set data_type */  
   if(data->data_len) 
     {
       data_type = xrec_data->type;
     }
-  XNEE_DEBUG ( (stderr ,"  -- xnee_record_dispatch() at 3 \n"  ));
-  
-  if ( xd->xnee_info->loops_left > 0 )
-    {
-      xd->xnee_info->loops_left--;
-    }
   
   
   switch(data->category)
@@ -314,38 +318,42 @@ xnee_human_dispatch(XPointer xpoint_xnee_data,
   
   XNEE_DEBUG ( (stderr ," --> xnee_human_dispatch()  \n"  ));
   
-  if (!data->data)
+  if ( (!data->data) || (data==NULL) )
     {
+      XRecordFreeData(data);
       XNEE_DEBUG ( (stderr ," <-- xnee_human_dispatch()  \n"  ));
+      xnee_process_count(XNEE_PROCESS_RESET);
       return;
+    } 
+  else
+    {
+      /*increment the process reply counter*/
+      xnee_process_count(XNEE_PROCESS_INC);
     }
   
-  
   /* fix by Ton van Vliet */
   xd = (xnee_data *) (xpoint_xnee_data) ;
   xrec_data = (XRecordDatum *) (data->data) ;
   
- 
-  if (xd->xnee_info->loops_left == 0 ) 
+  if ( xd->xnee_info->loops_left == 0 ) 
     {
-      XNEE_DEBUG ( (stderr ,"  -- xnee_human_dispatch() at 0.1 \n"  ));
-      xnee_close_down(xd);
-      exit(XNEE_OK);
+      XRecordFreeData(data);
+      return ;
+    } 
+  else
+    {
+      xd->xnee_info->loops_left--;
     }
- 
-  XNEE_DEBUG ( (stderr ,"  -- xnee_human_dispatch() at 2 \n"  ));
 
+ 
   /* if NOT XRECORDEndOffData or XRECORDClientDied  set data_type */  
   if(data->data_len) 
     {
       data_type = xrec_data->type;
     }
-  XNEE_DEBUG ( (stderr ,"  -- xnee_human_dispatch() at 3 \n"  ));
+
   
-  if ( xd->xnee_info->loops_left > 0 )
-    {
-      xd->xnee_info->loops_left--;
-    }
+
 
   switch(data->category)
     {
@@ -519,7 +527,7 @@ xnee_record_select_default_protocol (xnee_data *xd)
 int
 xnee_add_range_str (xnee_data *xd, int type, char *range)
 {
-  char start_str[10] ;
+  char start_str[XNEE_RANGE_STRING_SIZE] ;
   char *second;
   int start=0;
   int stop=0;
@@ -729,9 +737,6 @@ xnee_parse_range (xnee_data *xd,int type, char *range)
   int ret=0;
   int range_len=strlen(range);
 
-  printf ("error file=%s\n", xd->err_name);
-  printf ("error file=%d\n", xd->err_file);
-
   xnee_verbose ((xd,"int arg=%d\n", xd));
   xnee_verbose ((xd, "nt arg=%d\n", type));
   xnee_verbose ((xd, "string arg=%s\n", range));
@@ -862,8 +867,6 @@ xnee_setup_recording(xnee_data *xd)
   xd->record_setup->xids[0] = xnee_client_id (xd->control);
   xd->record_setup->xids[1] = xnee_client_id (xd->data);
   
-  xnee_verbose((xd, "EINAR 1\n"));
-
   /* 
    *Remove our clients displays from recording ...
    */
@@ -884,14 +887,15 @@ xnee_setup_recording(xnee_data *xd)
   xd->record_setup->rState=NULL; 
 */
 
-  xnee_verbose((xd, "\t  GetContext      0x%lx (%d clients intercepted))\n", xd->record_setup->rContext, (int) ( (xd->record_setup->rState) - (xd->record_setup->nclients) )));   
+  xnee_verbose((xd, "\t  GetContext      0x%lx (%d clients intercepted))\n", 
+		xd->record_setup->rContext, 
+		(int) ( (xd->record_setup->rState) - (xd->record_setup->nclients) )));   
   
   if (xd->xnee_info->interval != 0)
     {
       xnee_delay (xd->xnee_info->interval, "xnee:" );
     }
   
-  xnee_verbose((xd, "EINAR 2\n"));
   
   /* Enable context for async interception 
   XSynchronize(xd->data, True);  
@@ -905,6 +909,31 @@ xnee_setup_recording(xnee_data *xd)
 
 
 
+/**************************************************************
+ *                                                            *
+ * xnee_unsetup_recording                                     *
+ *                                                            *
+ *                                                            *
+ **************************************************************/
+int
+xnee_unsetup_recording(xnee_data *xd)
+{
+
+  xnee_verbose((xd, "---> xnee_unsetup_recording\n"));
+
+  xnee_verbose((xd, "---  disabling context \n"));
+  XRecordDisableContext(xd->control, xd->record_setup->rContext);
+
+  xnee_verbose((xd, "---  freeing context \n"));
+  XRecordFreeContext(xd->control, xd->record_setup->rContext);
+
+  xnee_verbose((xd, "<--- xnee_unsetup_recording\n"));
+  return (XNEE_OK);
+}
+
+
+
+
 /**************************************************************
  *                                                            *
  * xnee_record_loop                                           *
@@ -941,7 +970,7 @@ xnee_record_loop(xnee_data *xd)
 
 /**************************************************************
  *                                                            *
- * xnee_record_loop                                           *
+ * xnee_record_async                                          *
  *                                                            *
  *                                                            *
  **************************************************************/
@@ -959,16 +988,55 @@ xnee_record_async(xnee_data *xd)
 
 
   XRecordEnableContextAsync(xd->data, 
-		       xd->record_setup->rContext, 
-		       xd->rec_callback, 
-		       (XPointer) (xd) /* closure passed to Dispatch */);
-      
-  xnee_verbose((xd, " <--- xnee_record_async()\n"));
-  /*  while (1) 
+			    xd->record_setup->rContext, 
+			    xd->rec_callback, 
+			    (XPointer) (xd) /* closure passed to Dispatch */);
+  
+  while (1) 
     {
-      XRecordProcessReplies (xd->data); 
+      int ret;
+      /* has the user pressed a modifier+key */
+      if (xnee_check_km (xd)==XNEE_GRAB_DATA)
+	{
+	  xnee_verbose  ((xd," HANDLING \n"));
+	  ret = xnee_handle_km(xd) ; 
+	  if (ret == XNEE_GRAB_STOP)
+	    {
+	      xnee_verbose  ((xd," breaking async loop since STOP \n"));
+	      break;
+	    }
+	  else if (ret == XNEE_GRAB_RESUME)
+	    {
+	      xnee_verbose  ((xd," starting async loop since RESUME \n"));
+	      XRecordEnableContextAsync(xd->data, 
+					xd->record_setup->rContext, 
+					xd->rec_callback, 
+					(XPointer) (xd) /* closure passed to Dispatch */);
+	    }
+	}
+
+      /* handle data in the RECORD buffer */
+      xnee_process_replies(xd);
+      
+      if ( xd->xnee_info->loops_left == 0 ) 
+	{
+	  xnee_verbose  ((xd," closing down while loop in async loop\n"));
+	  break ; 
+	}
+
+      /* sleep for a little while 
+       * there might be someone wanting the CPU
+       */  
+      usleep (100*100);
     }
-  */
+
+  XRecordDisableContext(xd->control, 
+			xd->record_setup->rContext);
+  XRecordFreeContext(xd->control, 
+			xd->record_setup->rContext);
+  xnee_stop_session(xd);
+
+  xnee_verbose((xd, " <--- xnee_record_async()\n"));
   return (0);
 }
 
