@@ -30,6 +30,9 @@
 #include "libxnee/xnee_setget.h"
 #include "parse.h"
 
+int
+xnee_set_info(xnee_data *xd);
+
 
 /**************************************************************
  *                                                            *
@@ -53,19 +56,16 @@ int main(int argc,char *argv[])
     */
   xd = xnee_new_xnee_data();
 
-
   /*
-   * Init the structs. Use PACKAGE as a name for the program
+   * Init the structs. 
    */
-  xnee_init(xd, PACKAGE);
-
+  xnee_init(xd);
   
   /*
    * Init Recording variables
    * Since those are used when recording and replaying we 
    */
   xnee_record_init (xd);
-
   
   /*
    * Well .... parse the args
@@ -90,8 +90,8 @@ int main(int argc,char *argv[])
       if ( max > 0 )
 	{
 	  xnee_print_error ("Error:\n%s, amibigous ranges\n", PACKAGE);
-	  xnee_print_error (" You can't use --all_events or --everything\n");
-	  xnee_print_error (" together with --event_ranges ..... \n");
+	  xnee_print_error (" You can't use --all-events or --everything\n");
+	  xnee_print_error (" together with --event-ranges ..... \n");
 	  xnee_print_error (" For syntax description type %s --help\n",
 			    PACKAGE);
 
@@ -102,16 +102,6 @@ int main(int argc,char *argv[])
     }
 
 
-  /*
-   * If no recording client, init xnee_sync 
-   *
-   */
-  if ( ! xnee_is_recorder(xd) )
-    {
-      xnee_replay_init (xd, argv[0]);   
-    }
-  
-  
   /* 
    * Print settings 
    * only done if verbose mode  
@@ -127,7 +117,7 @@ int main(int argc,char *argv[])
       if (ret==XNEE_NO_PROT_CHOOSEN)
 	{
 	  xnee_print_error ("No protocol data choosen to record\n");
-	  xnee_print_error ("You can try out with xnee --all_events\n");
+	  xnee_print_error ("You can try out with xnee --all-events\n");
 	  exit(ret);
 	}
     }
@@ -140,6 +130,25 @@ int main(int argc,char *argv[])
   xnee_setup_display (xd);
   
 
+
+  /*
+   * If no recording client, init xnee_sync 
+   *
+   */
+  if ( ! xnee_is_recorder(xd) )
+    {
+      xnee_replay_init (xd);   
+    }
+  
+  
+  /*
+   * Save repeat mode so we can reset it after we are done
+   *
+   */
+  xnee_set_autorepeat (xd);
+  
+
+
   /*
    * are we recording or are we replaying
    */
@@ -150,8 +159,8 @@ int main(int argc,char *argv[])
        * Print settings 
        * if verbose mode that is 
        */
-      xnee_record_print_record_range (xd, NULL);
       xnee_print_xnee_settings       (xd, NULL); 
+      xnee_record_print_record_range (xd, NULL);
 
 
       /*
@@ -178,7 +187,7 @@ int main(int argc,char *argv[])
 	{
 	  xnee_verbose ((xd, "Entering main loop( recorder)\n"));
 
-	  xnee_record_loop(xd);
+	  xnee_record_async(xd);
 	}
     }
   else 
@@ -226,9 +235,10 @@ int main(int argc,char *argv[])
     }
    */
 
+  xnee_reset_autorepeat (xd);
+
   xnee_close_down(xd);
   
-
   exit(XNEE_OK); 
 }
 
