@@ -26,6 +26,11 @@
 #include "libxnee/xnee.h"
 #include "libxnee/xnee_setget.h"
 #include "libxnee/xnee_record.h"
+#include "libxnee/xnee_resolution.h"
+#include "libxnee/xnee_resource.h"
+#include "libxnee/xnee_km.h"
+#include "libxnee/xnee_grab.h"
+#include "libxnee/xnee_fake.h"
 #include "libxnee/xnee_replay.h"
 #include "libxnee/datastrings.h"
 #include "parse.h"
@@ -35,7 +40,7 @@
 
 static char *help[] = {
   "--help, -h                     ", "Print this message", 
-  "--display,d  displayname       ", "X server to contact (default is localhost)", 
+  "--display,-d  displayname      ", "X server to contact (default is localhost)", 
   "--flags, -flags                ", "Prints all flags/options xnee accepts",
   "--file, -f <file_name>         ", "Read data from file file_name (default is stdin)", 
   "--out, -o <file_name>          ", "Redirect all Xnee data to file_name (default is stdout)", 
@@ -62,7 +67,12 @@ static char *help[] = {
   "--human-printouts, -hp         ", "Prints human readable" , 
   "--record, -rec                 ", "Set recording mode (default)" , 
   "--replay, -rep                 ", "Set replaying mode" , 
+  "--speed-percent, -sp           ", "Set replaying speed (percentage)",
   "--stop-key mod,key, -sk        ", "When pressing modifier mod and key key Xnee exits" , 
+  "--pause-key mod,key, -pk       ", "When pressing modifier mod and key key Xnee pauses its current action" , 
+  "--resume-key mod,key, -rk      ", "When pressing modifier mod and key key Xnee resumes its paused action" , 
+  "--insert-key mod,key, -ik      ", "When pressing modifier mod and key key Xnee inserts a META mark in the log file" , 
+  "--write-settings file          ", "Writes settings to a resource file",
   "--print-settings, -ps          ", "Prints Xnee settings and waits (for <ENTER>)", 
   "--print-event-names, -pens     ", "Prints X11 event number and name ", 
   "--print-event-name, -pen <ev>  ", "Prints X11 event number or name coresponding to ev", 
@@ -71,6 +81,9 @@ static char *help[] = {
   "--print-request-names, -prns   ", "Prints X11 request number and name ", 
   "--print-request-name, -prn <req> ", "Prints X11 request number or name  coresponding to req", 
   "--print-data-names, -pdn       ", "Prints X11 data number and name ", 
+  "--recorded-resolution res      ", "Resolution used when recording",
+  "--replay-resolution res        ", "Resolution to use when replaying",
+  "--no-resolution-adjustment     ", "Don't use resolution adjustment",
   "--manpage                      ", "Prints Xnee help text in format as used when generating man page", 
   "--distribute, -di <LIST>       ", "Distribute recorded or replayed events to LIST where LIST is comma separated list of displays",
   "--device-event-range, -devera  <X_LIST> ", "Set device event range to X_LIST", 
@@ -82,7 +95,9 @@ static char *help[] = {
   "--extension-request-mainor-range, -erqmir  <X_LIST> ", "Set extension request minor range to X_LIST", 
   "--extension-reply-major-range, -erpmar  <X_LIST>    ","Set extension reply major range to X_LIST", 
   "--extension-reply-mainor-range, -erpmir  <X_LIST>   ","Set extension reply minor range to X_LIST",
-  "--force-replay, -fp             ", "Keep replaying even if we are out of sync .... dangerous",
+  "--retype-file <file>, -rf      ", "Types (fakes) the content of the specified file",
+  "--type-help, -tp               ", "Type this help message using faked keys (used to test xnee itself)",
+  "--force-replay, -fp            ", "Keep replaying even if we are out of sync .... dangerous",
   NULL 
 };
 
@@ -116,7 +131,7 @@ static char *description[] = {
 
 
 
-
+/*
 static char *obsolete_help[] = {
   "--help, -h                     ", "Print this message", 
   "--display,d  displayname       ", "X server to contact (default is localhost)", 
@@ -159,6 +174,11 @@ static char *obsolete_help[] = {
   "--extension_reply_mainor_range, -erpmir  <X_LIST>   ","Set extension reply minor range to X_LIST",
   NULL 
 };
+*/
+
+
+static int
+xnee_type_help (xnee_data *xd);
 
 
 /**************************************************************
@@ -195,6 +215,66 @@ xnee_parse_args (xnee_data* xd , int argc, char **argv )
 	  xnee_verbose((xd, "display=%s\n", xnee_get_display_name (xd)));
 	  continue;
 	}
+      else if (xnee_check (argv[i], "--recorded-resolution", "-rr"  ) )
+	{
+	  if (++i >= argc) 
+	    {
+	      xnee_usage(stderr);
+	      xnee_close_down(xd);
+	      exit(XNEE_WRONG_PARAMS);
+	    }
+	  if ( xnee_set_rec_resolution (xd, argv[i]))
+	    {
+	      xnee_verbose ((xd, "failed to set recorded resolution\n"));
+	      xnee_close_down(xd);
+	      exit(XNEE_BAD_RESOLTION );
+	    }
+	  xnee_verbose ((xd, "recored resolution= %dx%d\n", 
+			 xnee_get_rec_resolution_x(xd),
+			 xnee_get_rec_resolution_y(xd)));
+ 	  continue;
+	}
+      else if (xnee_check (argv[i], "--speed-percent", "-sp"  ) )
+	{
+	  if (++i >= argc) 
+	    {
+	      xnee_usage(stderr);
+	      xnee_close_down(xd);
+	      exit(XNEE_WRONG_PARAMS);
+	    }
+	  if ( xnee_set_replay_speed_str (xd, argv[i]))
+	    {
+	      xnee_verbose ((xd, "failed to set replay speed\n"));
+	      xnee_close_down(xd);
+	      exit(XNEE_BAD_RESOLTION );
+	    }
+	  xnee_verbose ((xd, "replaying speed= %d\n", 
+			 xnee_get_replay_speed(xd)));
+ 	  continue;
+	}
+      else if (xnee_check (argv[i], "--no-resolution-adjustment", "-rnra"  ) )
+	{
+	  xnee_unset_resolution_used (xd);
+	}
+      else if (xnee_check (argv[i], "--replay-resolution", "-rr"  ) )
+	{
+	  if (++i >= argc) 
+	    {
+	      xnee_usage(stderr);
+	      xnee_close_down(xd);
+	      exit(XNEE_WRONG_PARAMS);
+	    }
+	  if ( xnee_set_rep_resolution (xd, argv[i]))
+	    {
+	      xnee_verbose ((xd, "failed to set replay resolution\n"));
+	      xnee_close_down(xd);
+	      exit(XNEE_BAD_RESOLTION );
+	    }
+	  xnee_verbose ((xd, "replay resolution= %dx%d\n", 
+			 xnee_get_rep_resolution_x(xd),
+			 xnee_get_rep_resolution_y(xd)));
+ 	  continue;
+	}
       else if(xnee_check(argv[i], "--err", "-e" )) 
 	{
 	  if (++i >= argc)
@@ -269,10 +349,32 @@ xnee_parse_args (xnee_data* xd , int argc, char **argv )
 	      xnee_print_error ("Unable to open plugin file (%s)\n", argv[i]);
 	    }
 	}
+      else if(xnee_check(argv[i], "--retype-file", "-rf" )) 
+	{
+	  if (++i >= argc)
+	    {
+	      xnee_usage(stderr);
+	      xnee_close_down(xd);
+	      exit(XNEE_WRONG_PARAMS);
+	    }
+	  else
+	    {
+	      xnee_set_retyper(xd);
+	    }
+	  if ( xnee_type_file(xd, argv[i]) != 0 )
+	    {
+	      xnee_print_error ("Unable to open file \"%s\" for retyping \n", 
+				argv[i]);
+	    }
+	}
       else if(xnee_check(argv[i], "--all-clients", "-ac" )) 
 	{
 	  xnee_set_all_clients(xd);
 	}
+      else if(xnee_check(argv[i], "--type-help", "-th" )) 
+	{
+	  xnee_type_help(xd);
+	}
       else if(xnee_check(argv[i], "--future-clients", "-fc" )) 
 	{
 	  xnee_unset_all_clients(xd);
@@ -392,6 +494,11 @@ xnee_parse_args (xnee_data* xd , int argc, char **argv )
 	  xnee_verbose ((xd, "Found -rk/--resume ... : %s\n", argv[i]));
 	  xnee_grab_key (xd, XNEE_GRAB_RESUME, argv[++i]);
 	}
+      else if(xnee_check(argv[i], "--insert-key", "-ik")) 
+	{
+	  xnee_verbose ((xd, "Found -ik/--insert ... : %s\n", argv[i]));
+	  xnee_grab_key (xd, XNEE_GRAB_INSERT, argv[++i]);
+	}
       else if(xnee_check(argv[i], "--time", "-t")) 
 	{
 	  xnee_verbose ((xd, "CHECK ME ...buffer overflow ..... --loops\n"));
@@ -592,7 +699,7 @@ xnee_parse_args (xnee_data* xd , int argc, char **argv )
       else if(xnee_check(argv[i], "--version", "-V")) 
 	{
 	  xnee_version(xd);
-	  exit(0);
+	  exit(XNEE_OK);
 	}
       else if(xnee_check(argv[i], "--distribute", "-di")) 
 	{
@@ -621,6 +728,28 @@ xnee_parse_args (xnee_data* xd , int argc, char **argv )
 			xnee_get_data_file (xd)));
 	  continue;
 	}
+      else if ( xnee_check (argv[i], "--write-settings", "-ws"))
+	{
+	  if (++i >= argc)
+	    {
+	      xnee_usage(stderr);
+	      xnee_close_down(xd);
+	      exit(XNEE_WRONG_PARAMS);
+	    }
+	  if (xnee_check (argv[i], "stdout", "STDOUT"))
+	    xnee_write_settings_to_file (xd, stdout);
+	  else if (xnee_check (argv[i], "stderr", "STDERR"))
+	    xnee_write_settings_to_file (xd, stderr);
+	  else
+	    {
+	      FILE *file;
+	      file = fopen (argv[i],"w");
+	      xnee_write_settings_to_file (xd, 
+					   file);
+	      fclose(file);
+	    }
+	  exit(XNEE_OK);
+	}
       /* 
        * TO KEEP COMPATIBILITY WITH OLD 
        * - SCRIPTS USING XNEE
@@ -836,13 +965,9 @@ xnee_usage (FILE *fd)
 	  
 	  if (descr!=NULL)
 	    fprintf (fd, "\t%s\n\n", *cpp);
-	  else 
-	    ;
 	}
-      else 
-	;
     }
-  fprintf (fd, "\n  Report bugs to %s\n", XNEE_MAIL);
+  fprintf (fd, "\n  Report bugs to %s\n", XNEE_BUG_MAIL);
   
 }
 
@@ -917,18 +1042,14 @@ xnee_manpage (FILE *fd)
 	  
 	  if (descr!=NULL)
 	    fprintf (fd, "\n%s\n", *cpp);
-	  else 
-	    ;
 	}
-      else 
-	;
     }
 
   fprintf (fd ,".SH \"AUTHOR\" \n");
   fprintf (fd ,"Henrik Sandklef.\n");
 
   fprintf (fd ,".SH \"REPORTING BUGS\"\n");
-  fprintf (fd ,"Report bugs in the program to "XNEE_MAIL". \n");
+  fprintf (fd ,"Report bugs in the program to "XNEE_BUG_MAIL". \n");
 
   fprintf (fd ,".SH \"COPYRIGHT\"\n");
   fprintf (fd ,"Copyright (C) 2002 Henrik Sandklef.\n");
@@ -948,3 +1069,56 @@ xnee_manpage (FILE *fd)
   fprintf (fd ,"Mail corrections and additions to " XNEE_BUG_MAIL "\n");
 }
 
+int
+xnee_type_help (xnee_data *xd)
+{
+  int i ;
+  xnee_key_code x_kc;
+
+  char my_string[500];
+  char **cpp;
+
+  KeyCode shift_kc ;
+  KeyCode return_kc ;
+
+  xnee_verbose ((xd,"---> xnee_type_help\n"));
+  xnee_setup_display (xd);
+  xnee_replay_init (xd);   
+  xnee_set_autorepeat (xd);
+  if (!xnee_has_xtest_extension(xd))
+    exit(XNEE_NO_TEST_EXT);
+
+  shift_kc  = xnee_str2keycode (xd, "Shift_L");
+  return_kc = xnee_str2keycode (xd, "Return");
+  
+  for (cpp = help; *cpp; cpp+=1) 
+    {
+      xnee_verbose ((xd,"string to fake %s\n", *cpp));
+      strcpy(my_string,*cpp);
+      for (i=0;(size_t)i<strlen(my_string);i++)
+	{
+	  xnee_char2keycode(xd, my_string[i], &x_kc); 
+	  if (x_kc.shift_press)
+	    xnee_fake_key_event (xd, shift_kc, True,  CurrentTime);
+
+	  xnee_fake_key_event (xd, x_kc.kc, True,  CurrentTime);
+	  usleep (1000*100);
+	  xnee_fake_key_event (xd, x_kc.kc, False, CurrentTime);
+
+	  if (x_kc.shift_press)
+	      xnee_fake_key_event (xd, shift_kc, False,  CurrentTime);
+	    
+	}
+      usleep(1000*200);
+
+      xnee_fake_key_event (xd, return_kc, True,  CurrentTime);
+      xnee_fake_key_event (xd, return_kc, False, 100000);
+      
+      
+    } 
+ xnee_reset_autorepeat (xd);
+ xnee_verbose ((xd,"<--- xnee_type_help\n"));
+ 
+ xnee_close_down(xd);
+ exit(XNEE_OK);
+}
